/*
  SECON
  Secure Connection Script.
  Ensures secure (RC4) connection via HTTP (80) to exchange data with the server
  on the serverside PHP secon.php is required.

  how it works:
  1. connection status is determined.
  2. new connection is built up with Public Key Key Exchange procedure
  3. Data encryption with exchanged Key and RC4 encryption.
  4. Encryption Key is generated an renewed every period 
     (configuration: RefreshKeyTimeout)
  
  Data encryption to server:
  - By submitting forms (attach class secon to form, or adjust (see configuration)
  - By submitting Forms with Ajax
  - by Submitting Data directly with function secon_sendData
  - by Crypting Parameters in Links (attach class secon to HREF Links) 

*/

var SECON_PingTimeout   = 30; // in SEcons when a Ping is executed
var SECON_KeyRenewal    = 500; // in Seconds to renew the Key
var SECON_AESBLOCKSIZE  = 128; // Block (bit) Size
var SECON_AESKEYSIZE    = 256; // Bit Keysize 

function secon() {
      var C_debug = false;
      var ConKey = '';
      var ConKeyIV = '';
      var RSAPub = "";
      var Success = true;
      
      debug('Secon started');
      
      // Get Connection Key from Browser Windows Storage
      ConKey   = storage.get("ConKey");
      ConKeyIV = storage.get("ConKeyIV");
      debug("Active connection Key: " + ConKey);
      
      setInterval(function(){secon.Ping()}, (SECON_PingTimeout * 1000))
      setInterval(function(){secon.KeyRenewal()}, (SECON_KeyRenewal * 1000))

      /*
          Perform initial handshake  to exchange the Key
      */
      this.Ping = function(){
              debug("Ping: execution started.");
              ret = secon.SendData("PING - " + new Date().getTime() ,"ping");
              debug("Ping response: "+ret);
              if(ret){
                Success = true;
              }else{
                Success = false;
                //start complete new handshake
                secon.init();
            }
      }


      this.init = function(){
          if(document.location.href.indexOf("?") > -1) {
              document.location = document.location + '&seconm=init';
          } else {
              document.location = document.location + '?seconm=init';
          }
      }      
      
      /*
          Function To Attach handler to Forms
      */
      this.AttachForms = function(selector){
            // Attach to normal Forms --------------------------------------
            debug("Attaching secure handler to forms.");
            $( "form"+selector ).css("border", "solid 1px #FEA700");
              
            $( "form"+selector ).on( "submit", function( event ) {
            
              // Get Form Data
              debug("AttachForms: Form Serialization started.");
              var data = JSON.stringify($( this ).serializeArray());
              debug("Formdata: "+data);
              // Endcrypt Data
              hash = sha1(data);
              hash = Crypt(hash, ConKey, ConKeyIV);
              data = Crypt(data, ConKey, ConKeyIV);
              
              //Replace Form Data
              frmheight = $( this ).height();
              $( this ).height(frmheight);
              
              $( this ).html(' Formdata encrypted and is now transferred to server!<br><input type="hidden" name="secon" value="1"><input type="hidden" name="seconm" value="form"><input type="hidden" name="d" value="'+data+'"><input type="hidden" name="h" value="'+hash+'"><input type="hidden" name="secont" value="j">');
              // Form is sent normal
            });
      }
     /*
          Function To Attach Handler to Links
     */      
     this.AttachLinks = function(selector){
        debug("Attaching secure handler Links.");      
        $("a"+selector).each(function(){
          $(this).click(function(){
            var href = $( this ).attr('href');
            var qs = QueryStringToJSON(href)
            data = JSON.stringify(qs);
            hash = sha1(data);
            hash = Crypt(hash, ConKey, ConKeyIV);
            data = Crypt(data, ConKey, ConKeyIV);

            // now we evaluate the sending script
            var Qpos = href.indexOf('?');
            file = href.slice(0,Qpos)
            $( this ).attr('href',file + '?secon=1&seconm=get&d=' + encodeURIComponent(data) + '&h='+encodeURIComponent(hash)+"&secont=j");
          });
        });
    }
          
            

      /*
         Decrypt all recieved Elements by class
      */
      this.DeCryptContent = function(sel){
          debug('Starting decrypting content...');
          if(typeof ConKey === "undefined"){  
            alert('Secure Connection Alert!\nSession was lost. Please click ok for reconnection.');
            this.init();
           }
          $(sel).each(function(){
              data = $(this).html();
              data = Aes.Ctr.decrypt(data, ConKey, 256);
              $(this).html(data);
         });
         if($.mobile)
          $.mobile.activePage.trigger("create");
      }

      /*
          Perform initial handshake  to exchange the Key
      */
      this.Handshake = function(){
      Success = false;
      debug('Start initial handshake');
      $( document ).ready(function() {
         RSAPub = $('#RSAPub').html();
         debug('Public Key: '+RSAPub);
         
        
         // Crate random Key and send key crypted to server.
         var RSAPubI = RSA.getPublicKey(RSAPub);  
         ConKey     = CryptoJS.lib.WordArray.random(16);
         var salt = CryptoJS.lib.WordArray.random(SECON_AESBLOCKSIZE / 8);
         ConKey    = CryptoJS.PBKDF2(ConKey, salt, { keySize: 256/32, iterations: 500 });
         
         ConKey     = ConKey.toString(CryptoJS.enc.Base64)
         debug('Connection Key generated:'+ConKey);
         ConKeyHash = sha1(ConKey);
         debug('Connection Key hash:'+ConKeyHash);
         
         ConKeyIV      = CryptoJS.lib.WordArray.random(SECON_AESBLOCKSIZE / 8);
         ConKeyIV      = ConKeyIV.toString(CryptoJS.enc.Base64)
         debug('Connection Key IV:'+ConKeyIV);
         ConKeyIVHash  = sha1(ConKeyIV)
         debug('Connection Key IV Hash:'+ConKeyIVHash);
  
         ConKeyC = RSA.encrypt(ConKey, RSAPubI);
         debug('Connection Key crypted:'+ConKeyC);
         ConKeyHashC = RSA.encrypt(ConKeyHash, RSAPubI);
         debug('Connection Hash crypted:'+ConKeyHashC);
  
  
         ConKeyIVC = RSA.encrypt(ConKeyIV, RSAPubI);
         debug('Connection Key IV crypted:'+ConKeyIVC);
         ConKeyIVHashC = RSA.encrypt(ConKeyIVHash, RSAPubI);
         debug('Connection Hash IV crypted:'+ConKeyIVHashC);
  
         
         // Now we have to submit this to the server
         $.ajax({
              type: "POST",
              url: '?secon=1&seconm=handshake',
              data: { k:ConKeyC, h:ConKeyHashC, i:ConKeyIVC, ih:ConKeyIVHashC },
              success: function(data){
                debug('Server Response: '+data);
                if(data == 'OK'){
                    storage.set("ConKey", ConKey);
                    storage.set("ConKeyIV", ConKeyIV);
                    Success = true
                } else {
                    storage.set("ConKey", "");
                    storage.set("ConKeyIV", "");
                    Success = false
                }
              },
              async: false
            });
            debug('Handshake Success:' + Success)
            if(Success){
              // Now we have to renew the Connection Key (2084 Bit Encryption)
              Success = secon.KeyRenewal()
              var curLoc = document.location.toString();
              curLoc = curLoc.replace(/seconm=init/gi,'')  
              document.location = curLoc; 
            }
            return Success;
        });
     }
     
     /*
        Renew the Keys with the Server 
     */
     this.KeyRenewal = function(){
        debug("KeyRenewal, generating and crypting key...");
        var Data = [];
        var NewKey = CryptoJS.lib.WordArray.random(SECON_AESKEYSIZE / 8);
        var salt = CryptoJS.lib.WordArray.random(128/8);
        NewKey    = CryptoJS.PBKDF2(NewKey, salt, { keySize: 256/32, iterations: 500 });
        
        NewKey     = NewKey.toString(CryptoJS.enc.Base64)
        var NewKeyHash = sha1(NewKey);
        
        Data.push({ key : NewKey });
        debug("Key-Data:");
        debug(Data);
        
        ret = secon.SendJsonData(Data, 'krn');
        if(ret){
          debug('Key Renewal success.');
          ConKey = NewKey;
          debug('KeyRenewal, New ConKey:'+NewKey);
          Success = true;
        } else {
          ConKey = '';
          debug('Key Renewal failed!');
          Success = false;
        }
        storage.set("ConKey", ConKey);
     }

     /*
        Send crypted JSON Array to Server 
     */
     this.SendJsonData = function(JsonData, mode){
           // we add some salt ;-)
           JsonData.push( {secon : new Date().getTime() });
           var strData = JSON.stringify(JsonData);
           var Hash    = sha1(strData);
           
           debug('SendJsonData, Connection Key:'+ConKey);
           debug('SendJsonData, Data:'+strData);
           debug('SendJsonData, Hash:'+Hash);

           debug('SendJsonData, Crypting Data...');
           strDataC    = Crypt(strData, ConKey, ConKeyIV);
           debug('SendJsonData, Crypted Data: '+strDataC);
           //strDataC    = Base64.encode(strDataC);
           
           debug('SendJsonData, Crypting Hash...');
           HashC       = Crypt(Hash, ConKey, ConKeyIV);
           debug('SendJsonData, Crypted Hash: '+strDataC);
           //HashC       = Base64.encode(HashC);
           
           debug('SendJsonData, CryptedData:'+strDataC);
           debug('SendJsonData, CryptedHash:'+HashC);
           
           var ret = false;
           $.ajax({
                   type: "POST",
                   url: '?secon=1&seconm=' + mode +'&secont=j', //t=type (s=string, j=json)
                   data: { d: strDataC, h: HashC },
                   success: function(data){
                            debug('Server Response: '+data);
                            if(data == 'OK'){
                                ret = true
                              } else {
                                ret = false;
                                console.log("Data Transfer error: "+data);
                              }
                            },
                        async: false
                      });
          return ret;
     }
     
     /*
        Send crypted JSON Array to Server 
     */
     this.SendData = function(Data, mode){
           var strData = Data;
           var Hash    = sha1(strData);
           
           // Check if Connection key is set.
           if(typeof ConKey == 'undefined'){
            alert("Secure Connection broke down.\nClick OK to reconnect.");
            secon.init();
           }
           
           debug('SendData, Connection Key:'+ConKey);
           debug('SendData, Data:'+strData);
           debug('SendData, Hash:'+Hash);

           strDataC    = Crypt(strData, ConKey, ConKeyIV);
           //strDataC    = Base64.encode(strDataC);
           
           HashC       = Crypt(Hash, ConKey, ConKeyIV);
           //HashC       = Base64.encode(HashC);
           
           debug('SendJsonData, CryptedData:'+strDataC);
           debug('SendJsonData, CryptedHash:'+HashC);
           
           var ret = false;
           $.ajax({
                   type: "POST",
                   url: '?secon=1&seconm=' + mode +'&secont=s', //t=type (s=string, j=json)
                   data: { d: strDataC, h: HashC },
                   success: function(data){
                            debug('Server Response: '+data);
                            if(data == 'OK'){
                                ret = true;
                              } else {
                                ret = false;
                                console.log("Data Transfer error: "+data);
                              }
                            },
                        async: false
                      });
          return ret;
     }


    /* 
       QueryString string to Json conversion 
    */
    function QueryStringToJSON(str) {            
        var Qpos = str.indexOf('?')+1;
        str = str.slice(Qpos)
        var pairs = str.split('&');
        var result = {};
        pairs.forEach(function(pair) {
            pair = pair.split('=');
            result[pair[0]] = decodeURIComponent(pair[1] || '');
        });
        return JSON.parse(JSON.stringify(result));
    }
     
     /*
        Function to write to the console
     */
     function debug(msg){
      if(C_debug){
          console.log(msg);      
      }
     }
    function Crypt(msg, key, iv){
        debug("Crypt: Message:"+msg);
        debug("Crypt: Key:    "+key);
        var ciphertext = Aes.Ctr.encrypt(msg, key, 256);
        return ciphertext;
    }
    
}

 