<?php
/*        
   Secure Connection Class for PHP Applications and Browser Decryption / Encryption
   
    With this class you can transfer sensitive date with Port 80 with a client.
    Client does Encryption or Decryption and PHP Serverside as well.
    
    Start secon like this on the beginning of your PHP Script:
    
        require_once("secon/secon.cls.inc.php");
        $secon = new secon();
    
    Secon does now initialize a secure client connection.
    If you want to submit sensitive data, implement like this:
    
    echo $secon->CryptText('My Sensitive Data here');
    
    Please include the sensitive data into a div or span with a defined class: (<div class="seconCrypted"></div>)
    on the client side call the decryption process for all elements witht the defined class. 
    See example in secon.js
    
    Recieved POST / GET Data can be accessed like this:
    $secon->DATA
    
     
*/
$SECON_CONF["timeout"]  = 500; //Seconds to renew the session (forced reload)

$SECON_CONF["debug"]    = false; //Only enable in local DEV-Environment!
                                 // Critical Session keys are writte to the log 

$SECON_CONF["logfile"]  = './secon.log';  // Logfile to write down logs

class secon{
    private  $RSAPub;
    private  $RSAPriv;
    private  $ConKey;
    private  $ConKeyIV;
    private  $LastCon;
    public   $DATA;
    
    function __construct() {
      global $SECON_CONF;
      $this->Load();
      
      // Check if secon active
      $secActive = (@$_GET["secon"] == 1 || @$_POST["secon"])?true:false;
      $secMode   = (!empty($_POST["seconm"])) ? @$_POST["seconm"] : @$_GET["seconm"];
      $this->debug("--------------------- NEW REQUEST, Mode: $secMode ----------------");
            
      switch($secMode){
          case 'init'     : $this->Handshake1();
                            exit;
                            break;      
          case 'handshake': $this->Handshake2();
                            exit;
                            break;
          
          case 'krn':       $this->KeyRenewal();
                            exit;
                            break;

          case 'ping':      $this->Ping();
                            exit;
                            break;
          case 'form':     $this->ProcessForm('POST');
                           break;

          case 'get':     $this->ProcessForm('GET');
                           break;
          
      }
      
      // Check if a full reconnect is necessary.
      if ( ($this->LastCon == 0)  ||
           (empty($this->ConKey)) ||
           ($this->LastCon < (time() - $SECON_CONF["timeout"]))){
             // Start a news Secure Session - handshake
             $this->debug("New Handshake: LasCon: ".@$this->LastCon.", ConKey:".@$this->Conkey);
             $this->Handshake1();
           }
      }

   
    /*
      Perform a Handshake to exchange and generate keys.
    */
    private function Handshake1(){
            $this->debug("Perform Handshake1");
            $this->debug("Generating Private and public keys.");
            
            set_include_path("./secon/phpseclib");
            include('Crypt/RSA.php');
            $rsa = new Crypt_RSA();  
    
            // we have to generate a public and private key.
            $keys = ($rsa->createKey()); // == $rsa->createKey(1024) where 1024 is the key size
            $this->RSAPriv = $keys["privatekey"];
            $this->RSAPub  = $keys["publickey"];
            $this->debug('Private and Public Key generated.');
            $this->debug('Public Key: '.$this->RSAPub);
            $this->debug('Private Key: '.$this->RSAPriv);
            ?>
              <html><head>
                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
                <script type="text/javascript" src="secon/js/jquery_min.js"></script>            <!--- JQUERY -->
                <script type="text/javascript" src="secon/js/pbkdf2.js"></script> <!-- Used for Key Generation and Salting --->  
                
                <script type="text/javascript" src="secon/js/jsbn.js"></script>   <!-- BigInt etc. Java Helper Tools --->
                <script type="text/javascript" src="secon/js/rsa2.js"></script>   <!-- RSA Encryption --->
                <script type="text/javascript" src="secon/js/sha1.js"></script>   <!-- Used SHA1 Hashing --->
                <script type="text/javascript" src="secon/js/aes.js"></script>    <!-- AES Ecnryption --->
                
                <script type="text/javascript" src="secon/js/storage.js"></script> <!-- Browser Window Storage of Variables and Content --->
                <script type="text/javascript" src="secon/js/secon.js"></script>   <!-- Secure Connection Class --->

                <script type="text/javascript">
                    var secon = new secon();    // Init new Secon Class.
                    secon.Handshake();          // Start the Handshake Procedure...
                </script>
              </head>
              <body>
                 <i>Secon is establishing secure client server connection....</i>
                <div id="RSAPub" style="display: none;"><?php echo $this->RSAPub; ?></div>
                <div id="secon_status""></div>
              </body>
            </html>
            <?php
            $this->save();
            exit;                       
    }
    
    /*
      Renew Data Transfer Keys
    */
    private function KeyRenewal(){
         $data = $this->GetSeconData();
         $data = $data[0];
         if(!empty($data["key"])){
            $this->ConKey = $data["key"];
            $this->Save();
            $this->debug("KeyRenewal: Success, Key: ".$this->ConKey);
            echo "OK";
            exit; 
         } else {
            echo "ERROR KeyExchange!";
            $this->debug("Key Exchange failed!");
            exit;
         }
    }
    
    /*
      Perform a Ping
    */
    private function Ping(){
      $data = $this->GetSeconData();
      if(substr($data,0,4) == "PING"){
        echo "OK";
        $this->debug("PING OK!");
      } else {
      $this->debug("PING ERRROR");
        echo "ERROR Ping not correct: ".substr($data,0,4);
      }
      $this->Save();    
    }    
    
    /*
      Get Crypted Data Transfer
    */
    private function GetSeconData(){
        global $_POST;
        global $_GET;
        $d   = (!empty($_POST["d"])) ? $_POST["d"] : $_GET["d"];
        $h   = (!empty($_POST["h"])) ? $_POST["h"] : $_GET["h"];
        $t = (!empty($_POST["secont"]))? $_POST["secont"] : $_GET["secont"];
        
        $this->debug("GetSeconData: Connection Key:".$this->ConKey);
        $this->debug("GetSeconData: Data: $d");
        $this->debug("GetSeconData: Hash: $h");
        
        $d = $this->DeCrypt($d, $this->ConKey, $this->ConKeyIV);
        $h = $this->DeCrypt($h, $this->ConKey, $this->ConKeyIV);
        
        $hCalc = sha1(utf8_decode($d));
        
//        $this->debug("GetSeconData: Data: ".print_r($d,true));

        if($h != $hCalc){
          $this->debug("GetSeconData: hash does not match!, Recieved Hash: $h, Calculated Hash: $hCalc"); 
          return false;
        } else {
            $this->debug("GetSeconData: Hash matches - OK.");
          if($t == 'j'){
              $this->debug("GetSeconData: Json DataType.");
              $da =  $this->JsonToArray($d);
              return  $da; 
          }elseif($t == 's'){
              $this->debug("GetSeconData: String DataType.");
              return $d;          
          }
        }
            
    
    }
    
    /*
      Perform a Handshake to exchange and generate keys.
    */
    private function Handshake2(){
        global $_POST;
        $hash = $_POST["h"];
        $key  = $_POST["k"];
        $iv      = $_POST["i"];
        $ivhash  = $_POST["ih"];
        
        $this->debug("Handshake2, PrivateKey: $this->RSAPriv");
        $this->debug("Handshake2, Hash: $hash");
        $this->debug("Handshake2, Key: $key");
        $this->debug("Handshake2, IV: $iv");
        $this->debug("Handshake2, IV Hash: $ivhash");

        
        $hash = base64_decode($hash);
        $key = base64_decode($key);
        $ivhash = base64_decode($ivhash);
        $iv = base64_decode($iv);

        openssl_private_decrypt($key,  $KeyDecrypted, openssl_pkey_get_private($this->RSAPriv));
        openssl_private_decrypt($hash, $HashDecrypted, openssl_pkey_get_private($this->RSAPriv));
        openssl_private_decrypt($iv,  $IVDecrypted, openssl_pkey_get_private($this->RSAPriv));
        openssl_private_decrypt($ivhash, $IVHashDecrypted, openssl_pkey_get_private($this->RSAPriv));

        $this->debug("Handshake2, DecryptedKey: $KeyDecrypted");
        $this->debug("Handshake2, DecryptedHash: $HashDecrypted");
        $this->debug("Handshake2, DecryptedIV: $IVDecrypted");
        $this->debug("Handshake2, DecryptedIVHash: $IVHashDecrypted");

              
        $HashLocal = sha1($KeyDecrypted);
        $IVHashLocal = sha1($IVDecrypted);
        if(($HashLocal == $HashDecrypted) && ($IVHashLocal == $IVHashDecrypted)){
          $this->debug("Handshake2, Hash and Key verified.");
          echo "OK";
          $this->LastCon = time();
          $this->ConKey   = $KeyDecrypted;
          $this->debug("New connection Key: ". $this->ConKey);
          $this->ConKeyIV = $IVDecrypted;
          $this->debug("New connection IV: ".$this->ConKeyIV);
        } else {
          $this->debug("Handshake2, Hash and Key verification ERROR.");
          echo "ERROR";
        }
        $this->save();
    }        
    
    /* 
      Loads data from the session.
    */     
    private function Load(){
          global $_SESSION;
          session_start();
/*          session_register("PRIVKEY");
          session_register("PUBKEY");
          session_register("COMKEY");
          session_register("COMKEYIV");
          session_register("COMKEY_AGE");
          session_register("LASTPING");
*/          
          $this->RSAPriv  = @$_SESSION["PRIVKEY"];
          $this->RSAPub   = @$_SESSION["PUBKEY"]; 
          $this->ConKey   = @$_SESSION["COMKEY"];
          $this->ConKeyIV = @$_SESSION["COMKEYIV"];
          $this->LastCon  = intval(@$_SESSION["LASTCON"]);
    }

    /* 
     Stores the important data in the Session
    */     
    private function Save(){
      global $_SESSION;
      $_SESSION["PRIVKEY"]   = $this->RSAPriv;
      $_SESSION["PUBKEY"]    = $this->RSAPub; 
      $_SESSION["COMKEY"]    = $this->ConKey;
      $_SESSION["COMKEYIV"]  = $this->ConKeyIV;
      $_SESSION["LASTCON"]   = time();
    } 


    private function JsonToArray($json){
      $data = json_decode($json, true);
      $this->debug("JsonToArray: Data: ".print_r($data,true));

//      if(count($data) == 1){
//          return $data[0];   
//       } else {
          return $data;
//       }
    }    

    /*
      Wrapper Function to Decrypt Data
   */
    private function Decrypt($data, $key, $iv){
        $this->debug("Decrypt: Data: $data");
        $this->debug("Decrypt: Key:  $key");

        $plaintext = AesCtr::decrypt($data, $key, 256);
         
        $this->debug("Decrpt: Data: $plaintext");
        return $plaintext;
    }

    /* 
     Debug class active?
    */     
    private function debug($msg){
      global $SECON_CONF;
      if($SECON_CONF["debug"]){
        file_put_contents($SECON_CONF["logfile"], date('Y-m-d H:i:s').": ".$msg."\n", FILE_APPEND);
        }
    }
    
    /*
      Process Forms which were crypted sent and provide data in 
      public Array DATA
    */
    private function ProcessForm($mode){
        $this->debug("ProcessForm");
        $data =  $this->GetSeconData();
        $this->debug("ProcessForm: Recieved Data:".print_r($data,true));
        if(count($data) > 0 ){
            if($mode == 'POST'){
              $this->debug("ProcessForm: Mode=POST");
              if(is_array($data)){
                foreach($data as $entry){
                  $this->DATA[$entry["name"]] = $entry["value"];
                }
              }
            }elseif($mode == 'GET'){
              $this->debug("ProcessForm: Mode=GET");
              if(is_array($data))
                  foreach($data as $key => $value){
                    $this->DATA[$key] = $value;
                  }
            }
          }
      $this->debug("ProcessForm: DATA:".print_r($this->DATA,true)); 
    }
    
    /* 
     Crypt Text with current key
    */     
    public function CryptText($msg){
        $key = $this->ConKey;
        $this->debug("CryptText: Data: $msg");
        $this->debug("CryptText: Key:  $key");
        $crypted = AesCtr::encrypt($msg, $key, 256);
        $this->debug("CryptText: Crypted:  $crypted");
        return $crypted;
    }    
}


/* 
 UTF-8 Helper Functions
*/     
function mb_chr($char) {
  return mb_convert_encoding('&#'.intval($char).';', 'UTF-8', 'HTML-ENTITIES');
}
function mb_ord($char) {
    $result = unpack('N', mb_convert_encoding($char, 'UCS-4BE', 'UTF-8'));
    
    if (is_array($result) === true) {
    return $result[1];
    }
    return ord($char);
}
require_once("secon/aes.cls.inc.php");
?>